import json
import sys

from geniusweb.connection.ConnectionEnd import ConnectionEnd 
from geniusweb.inform.Inform import Inform
from geniusweb.actions.Action import Action
from tudelft.utilities.listener.DefaultListenable import DefaultListenable
from packson3.ObjectMapper import ObjectMapper
from tudelft.utilities.listener.Listener import Listener

class StdInOutConnectionEnd(DefaultListenable[Inform],ConnectionEnd[Inform, Action]):
	'''
	A connectionend for a party that forwards stdin to the party,
	and pushes outdata from the party back into the stream.
	'''
	def __init__(self):
		super().__init__()
		self._jackson = ObjectMapper()

	
	def send(self,act:Action ):
		data=json.dumps(self._jackson.toJson(act))
		sys.stdout.write(len(data).to_bytes(4,'little'))
		sys.stdout.write(bytes(data, 'utf-8'))
	
	def run(self, receiver: Listener[Inform]):
		'''
		@param receiver the listener for the information
		runs until the input breaks.
		returns only after termination 
		@raise exception if the read or the receiver throws: 
		'''
		# blocking read
		while True:
			data=self.readblock()
			if not data: 
				break
			
			receiver.notifyChange(data)
		
	def readblock(self)->bytearray:
		'''
		@return next block of data from stdin, or None if stream closed.
		'''
		sizedata=sys.stdin.buffer.raw.read(4)
		if len(sizedata)<4:
			return None
		remaining=int.from_bytes(sizedata, byteorder='little',signed=True)
		
		data=bytearray()
		while remaining>0:
			newdata=sys.stdin.buffer.raw.read(remaining)
			if len(newdata)==0:
				return None
			data.extend(bytearray(newdata))
			remaining=remaining-len(newdata)
		return data
	