from geniusweb.references.Reference import Reference
from uri import URI

class ProfileRef (Reference):
	'''
	A URI reference to get a copy of a profile. This usually is a "ws://"
	(websocket) URI, allowing users (parties) to get notifications if the profile
	is changed.
	<p>
	In debugging scenarios, this is usually a "file://" URI, in which case the
	profile is assumed to be static during the run. Parties that support
	debugging eg with the simplerunner should support the file: protocol.
	'''

	def __init__(self, profile:URI):
		if profile == None:
			raise ValueError("profile=null")
		self._profile = profile
		
	def getURI(self) -> URI :
		return self._profile

	def __str__(self):
		return "ProfileRef[" + self._profile + "]"

	def __eq__(self, other):
		return super().__eq__(other) and isinstance(other, self.__class__) and \
			self._name==other._name

	def __hash__(self):
		'''
		support for using this in dict etc
		'''
		return hash(self._profile)
